@@ -15,13 +15,16 @@ import operator
 import os
 import shutil
 
-import pybedtools
+import numpy
+try:
+    import pybedtools
+except ImportError:
+    pybedtools = None
 import pysam
-from py_descriptive_statistics import Enum as Stats
 
 from bcbio import bam, broad, utils
 from bcbio.log import logger
-from bcbio.distributed.messaging import parallel_runner, zeromq_aware_logging
+from bcbio.distributed import multi, prun
 from bcbio.distributed.split import parallel_split_combine
 from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import shared
@@ -32,11 +35,12 @@ def parallel_callable_loci(in_bam, ref_file, config):
     config["algorithm"]["memory_adjust"] = {"direction": "decrease", "magnitude": 2}
     data = {"work_bam": in_bam, "sam_ref": ref_file, "config": config}
     parallel = {"type": "local", "cores": num_cores, "module": "bcbio.distributed"}
-    runner = parallel_runner(parallel, {}, config)
-    split_fn = shared.process_bam_by_chromosome("-callable.bed", "work_bam")
-    out = parallel_split_combine([[data]], split_fn, runner,
-                                 "calc_callable_loci", "combine_bed",
-                                 "callable_bed", ["config"])[0]
+    items = [[data]]
+    with prun.start(parallel, items, config) as runner:
+        split_fn = shared.process_bam_by_chromosome("-callable.bed", "work_bam")
+        out = parallel_split_combine(items, split_fn, runner,
+                                     "calc_callable_loci", "combine_bed",
+                                     "callable_bed", ["config"])[0]
     return out[0]["callable_bed"]
 
 def combine_bed(in_files, out_file, config):
@@ -50,7 +54,7 @@ def combine_bed(in_files, out_file, config):
                         shutil.copyfileobj(in_handle, out_handle)
     return out_file
 
-@zeromq_aware_logging
+@multi.zeromq_aware_logging
 def calc_callable_loci(data, region=None, out_file=None):
     """Determine callable bases for input BAM using Broad's CallableLoci walker.
 
@@ -243,14 +247,13 @@ def _analysis_block_stats(regions):
     def descriptive_stats(xs):
         if len(xs) < 2:
             return xs
-        calc = Stats(xs)
         parts = ["min: %s" % min(xs),
-                 "5%%: %s" % calc.percentile(5),
-                 "25%%: %s" % calc.percentile(25),
-                 "median: %s" % calc.percentile(50),
-                 "75%%: %s" % calc.percentile(75),
-                 "95%%: %s" % calc.percentile(95),
-                 "99%%: %s" % calc.percentile(99),
+                 "5%%: %s" % numpy.percentile(xs, 5),
+                 "25%%: %s" % numpy.percentile(xs, 25),
+                 "median: %s" % numpy.percentile(xs, 50),
+                 "75%%: %s" % numpy.percentile(xs, 75),
+                 "95%%: %s" % numpy.percentile(xs, 95),
+                 "99%%: %s" % numpy.percentile(xs, 99),
                  "max: %s" % max(xs)]
         return "\n".join(["  " + x for x in parts])
     logger.info("Identified %s parallel analysis blocks\n" % len(region_sizes) +
