@@ -4,10 +4,9 @@ import contextlib
 
 from IPython.parallel import require
 
-from bcbio.distributed import ipython
-from bcbio.ngsalign import alignprep, tophat, star
-from bcbio.pipeline import (disambiguate, sample, lane, qcsummary, shared,
-                            variation, rnaseq, alignment)
+from bcbio.ngsalign import alignprep
+from bcbio.pipeline import (config_utils, disambiguate, sample, lane, qcsummary, shared,
+                            variation, rnaseq)
 from bcbio.provenance import system
 from bcbio import structural
 from bcbio import chipseq
@@ -21,10 +20,10 @@ def _setup_logging(args):
     if len(args) == 1 and isinstance(args[0], (list, tuple)):
         args = args[0]
     for arg in args:
-        if ipython.is_nested_config_arg(arg):
+        if config_utils.is_nested_config_arg(arg):
             config = arg["config"]
             break
-        elif ipython.is_std_config_arg(arg):
+        elif config_utils.is_std_config_arg(arg):
             config = arg
             break
     if config is None:
@@ -43,7 +42,6 @@ def _setup_logging(args):
 def process_lane(*args):
     with _setup_logging(args):
         return apply(lane.process_lane, *args)
-process_lane.metadata = {"resources": ["picard"]}
 
 @require(lane)
 def trim_lane(*args):
@@ -54,16 +52,6 @@ def trim_lane(*args):
 def process_alignment(*args):
     with _setup_logging(args):
         return apply(lane.process_alignment, *args)
-process_alignment.metadata = {"resources": ["star", "novoalign", "bwa", "bowtie2",
-                                            "tophat2", "bowtie", "tophat"],
-                              "ensure": {"tophat": tophat.job_requirements,
-                                         "tophat2": tophat.job_requirements,
-                                         "star": star.job_requirements}}
-
-@require(alignprep)
-def prep_align_inputs(*args):
-    with _setup_logging(args):
-        return apply(alignprep.create_inputs, *args)
 
 @require(alignprep)
 def prep_align_inputs(*args):
@@ -74,7 +62,6 @@ def prep_align_inputs(*args):
 def postprocess_alignment(*args):
     with _setup_logging(args):
         return apply(lane.postprocess_alignment, *args)
-postprocess_alignment.metadata = {"resources": ["gatk"]}
 
 @require(sample)
 def merge_sample(*args):
@@ -85,7 +72,6 @@ def merge_sample(*args):
 def delayed_bam_merge(*args):
     with _setup_logging(args):
         return apply(sample.delayed_bam_merge, *args)
-delayed_bam_merge.metadata = {"resources": ["samtools"]}
 
 @require(sample)
 def recalibrate_sample(*args):
@@ -96,7 +82,6 @@ def recalibrate_sample(*args):
 def prep_recal(*args):
     with _setup_logging(args):
         return apply(recalibrate.prep_recal, *args)
-prep_recal.metadata = {"resources": ["gatk"]}
 
 @require(recalibrate)
 def write_recal_bam(*args):
@@ -117,31 +102,26 @@ def split_variants_by_sample(*args):
 def piped_bamprep(*args):
     with _setup_logging(args):
         return apply(bamprep.piped_bamprep, *args)
-piped_bamprep.metadata = {"resources": ["gatk"]}
 
 @require(variation)
 def postprocess_variants(*args):
     with _setup_logging(args):
         return apply(variation.postprocess_variants, *args)
-postprocess_variants.metadata = {"resources": ["gatk-vqsr", "gatk", "snpEff"]}
 
 @require(qcsummary)
 def pipeline_summary(*args):
     with _setup_logging(args):
         return apply(qcsummary.pipeline_summary, *args)
-pipeline_summary.metadata = {"resources": ["gatk", "picard"]}
 
 @require(rnaseq)
 def generate_transcript_counts(*args):
     with _setup_logging(args):
         return apply(rnaseq.generate_transcript_counts, *args)
-generate_transcript_counts.metadata = {"resources": ["samtools","gatk"]}
 
 @require(rnaseq)
 def run_cufflinks(*args):
     with _setup_logging(args):
         return apply(rnaseq.run_cufflinks, *args)
-run_cufflinks.metadata = {"resources": ["cufflinks"]}
 
 @require(sample)
 def generate_bigwig(*args):
@@ -157,7 +137,6 @@ def combine_bam(*args):
 def variantcall_sample(*args):
     with _setup_logging(args):
         return apply(genotype.variantcall_sample, *args)
-variantcall_sample.metadata = {"resources": ["gatk", "freebayes", "gatk-haplotype"]}
 
 @require(vcfutils)
 def combine_variant_files(*args):
@@ -178,7 +157,6 @@ def merge_variant_files(*args):
 def prep_gemini_db(*args):
     with _setup_logging(args):
         return apply(population.prep_gemini_db, *args)
-prep_gemini_db.metadata = {"resources": ["gemini"]}
 
 @require(structural)
 def detect_sv(*args):
@@ -199,7 +177,6 @@ def compare_to_rm(*args):
 def coverage_summary(*args):
     with _setup_logging(args):
         return apply(coverage.summary, *args)
-coverage_summary.metadata = {"resources": ["bcbio_coverage"]}
 
 @require(disambiguate)
 def run_disambiguate(*args):
