@@ -2,9 +2,9 @@
 """
 from bcbio import structural, utils, chipseq
 from bcbio.bam import callable
-from bcbio.ngsalign import alignprep, tophat, star
+from bcbio.ngsalign import alignprep
 from bcbio.pipeline import (disambiguate, lane, qcsummary, sample, shared, variation,
-                            rnaseq, alignment)
+                            rnaseq)
 from bcbio.variation import (bamprep, coverage, realign, genotype, ensemble, multi, population,
                              recalibrate, validate, vcfutils)
 
@@ -19,13 +19,10 @@ def trim_lane(*args):
 @utils.map_wrap
 def process_alignment(*args):
     return lane.process_alignment(*args)
-process_alignment.metadata = {"resources": ["novoalign", "bwa", "bowtie2", "tophat2"],
-                              "ensure": {"tophat2": tophat.job_requirements}}
 
 @utils.map_wrap
 def postprocess_alignment(*args):
     return lane.postprocess_alignment(*args)
-postprocess_alignment.metadata = {"resources": ["gatk"]}
 
 @utils.map_wrap
 def prep_align_inputs(*args):
@@ -38,7 +35,6 @@ def merge_sample(*args):
 @utils.map_wrap
 def delayed_bam_merge(*args):
     return sample.delayed_bam_merge(*args)
-delayed_bam_merge.metadata = {"resources": ["samtools"]}
 
 @utils.map_wrap
 def piped_bamprep(*args):
@@ -47,7 +43,6 @@ def piped_bamprep(*args):
 @utils.map_wrap
 def prep_recal(*args):
     return recalibrate.prep_recal(*args)
-prep_recal.metadata = {"resources": ["gatk"]}
 
 @utils.map_wrap
 def write_recal_bam(*args):
@@ -64,7 +59,6 @@ def split_variants_by_sample(*args):
 @utils.map_wrap
 def postprocess_variants(*args):
     return variation.postprocess_variants(*args)
-postprocess_variants.metadata = {"resources": ["gatk-vqsr", "gatk", "snpEff"]}
 
 @utils.map_wrap
 def pipeline_summary(*args):
@@ -77,7 +71,6 @@ def generate_transcript_counts(*args):
 @utils.map_wrap
 def run_cufflinks(*args):
     return rnaseq.run_cufflinks(*args)
-run_cufflinks.metadata = {"resources": ["cufflinks"]}
 
 @utils.map_wrap
 def generate_bigwig(*args):
@@ -90,7 +83,6 @@ def combine_bam(*args):
 @utils.map_wrap
 def variantcall_sample(*args):
     return genotype.variantcall_sample(*args)
-variantcall_sample.metadata = {"resources": ["gatk", "freebayes", "gatk-haplotype"]}
 
 @utils.map_wrap
 def combine_variant_files(*args):
@@ -115,7 +107,6 @@ def combine_calls(*args):
 @utils.map_wrap
 def prep_gemini_db(*args):
     return population.prep_gemini_db(*args)
-prep_gemini_db.metadata = {"resources": ["gemini"]}
 
 @utils.map_wrap
 def combine_bed(*args):
@@ -132,7 +123,6 @@ def compare_to_rm(*args):
 @utils.map_wrap
 def coverage_summary(*args):
     return coverage.summary(*args)
-coverage_summary.metadata = {"resources": ["bcbio_coverage"]}
 
 @utils.map_wrap
 def run_disambiguate(*args):
