@@ -8,6 +8,7 @@ import contextlib
 from distutils.version import LooseVersion
 import os
 import shutil
+import string
 import subprocess
 import sys
 
@@ -17,12 +18,15 @@ import yaml
 from bcbio import utils
 from bcbio.pipeline import genome
 from bcbio.variation import effects
+from bcbio.provenance import programs
 
 REMOTES = {
     "requirements": "https://raw.github.com/chapmanb/bcbio-nextgen/master/requirements.txt",
     "gitrepo": "git://github.com/chapmanb/bcbio-nextgen.git",
     "cloudbiolinux": "https://github.com/chapmanb/cloudbiolinux.git",
-    "genome_resources": "https://raw.github.com/chapmanb/bcbio-nextgen/master/config/genomes/%s-resources.yaml"}
+    "genome_resources": "https://raw.github.com/chapmanb/bcbio-nextgen/master/config/genomes/%s-resources.yaml",
+    "snpeff_dl_url": ("http://downloads.sourceforge.net/project/snpeff/databases/v{snpeff_ver}/"
+                      "snpEff_v{snpeff_ver}_{genome}.zip")}
 
 def upgrade_bcbio(args):
     """Perform upgrade of bcbio to latest release, or from GitHub development version.
@@ -58,8 +62,38 @@ def upgrade_bcbio(args):
         print("  Add {t}/bin to PATH and {t}/lib to LD_LIBRARY_PATH".format(t=args.tooldir))
     save_install_defaults(args)
     args.datadir = _get_data_dir()
+    _install_container_bcbio_system(args.datadir)
     return args
 
+def _install_container_bcbio_system(datadir):
+    """Install limited bcbio_system.yaml file for setting core and memory usage.
+
+    Adds any non-specific programs to the exposed bcbio_system.yaml file, only
+    when upgrade happening inside a docker container.
+    """
+    base_file = os.path.join(datadir, "config", "bcbio_system.yaml")
+    if not os.path.exists(base_file):
+        return
+    expose_file = os.path.join(datadir, "galaxy", "bcbio_system.yaml")
+    expose = set(["memory", "cores", "jvm_opts"])
+    with open(base_file) as in_handle:
+        config = yaml.load(in_handle)
+    if os.path.exists(expose_file):
+        with open(expose_file) as in_handle:
+            expose_config = yaml.load(in_handle)
+    else:
+        expose_config = {"resources": {}}
+    for pname, vals in config["resources"].iteritems():
+        expose_vals = {}
+        for k, v in vals.iteritems():
+            if k in expose:
+                expose_vals[k] = v
+        if len(expose_vals) > 0 and pname not in expose_config["resources"]:
+            expose_config["resources"][pname] = expose_vals
+    with open(expose_file, "w") as out_handle:
+        yaml.dump(expose_config, out_handle, default_flow_style=False, allow_unicode=False)
+    return expose_file
+
 def _default_deploy_args(args):
     flavors = {"minimal": "ngs_pipeline_minimal",
                "full": "ngs_pipeline"}
@@ -86,18 +120,17 @@ def _update_conda_packages():
     pkgs = ["biopython", "boto", "cython", "ipython", "lxml", "matplotlib",
             "nose", "numpy", "pandas", "patsy", "pycrypto", "pip", "pysam",
             "pyyaml", "pyzmq", "requests", "scipy", "tornado", "statsmodels"]
+    channels = ["-c", "https://conda.binstar.org/faircloth-lab"]
     if os.path.exists(conda_bin):
-        subprocess.check_call([conda_bin, "install", "--yes"] + pkgs)
-        # Remove until can get 13.1.0 working cleanly on CentOS
-        #extra_pkgs = ["zeromq", "pyzmq"]
-        #binstar_user = "minrk"
-        #subprocess.check_call([conda_bin, "install", "--yes",
-        #                       "-c", "http://conda.binstar.org/%s" % binstar_user] + extra_pkgs)
+        subprocess.check_call([conda_bin, "install", "--yes", "numpy"])
+        subprocess.check_call([conda_bin, "install", "--yes"] + channels + pkgs)
 
 def _get_data_dir():
     base_dir = os.path.realpath(os.path.dirname(os.path.dirname(sys.executable)))
     if "anaconda" not in os.path.basename(base_dir) and "virtualenv" not in os.path.basename(base_dir):
-        raise ValueError("Cannot update data for bcbio-nextgen not installed by installer.")
+        raise ValueError("Cannot update data for bcbio-nextgen not installed by installer.\n"
+                         "bcbio-nextgen needs to be installed inside an anaconda environment \n"
+                         "located in the same directory as `galaxy` `genomes` and `gemini_data` directories.")
     return os.path.dirname(base_dir)
 
 def upgrade_bcbio_data(args, remotes):
@@ -115,9 +148,10 @@ def upgrade_bcbio_data(args, remotes):
     cbl_deploy.deploy(s)
     _upgrade_genome_resources(s["fabricrc_overrides"]["galaxy_home"],
                               remotes["genome_resources"])
-    _upgrade_snpeff_data(s["fabricrc_overrides"]["galaxy_home"], args)
+    _upgrade_snpeff_data(s["fabricrc_overrides"]["galaxy_home"], args, remotes)
     if 'data' in args.toolplus:
-        subprocess.check_call(["gemini", "update", "--dataonly"])
+        gemini = os.path.join(os.path.dirname(sys.executable), "gemini")
+        subprocess.check_call([gemini, "update", "--dataonly"])
 
 def _upgrade_genome_resources(galaxy_dir, base_url):
     """Retrieve latest version of genome resource YAML configuration files.
@@ -142,14 +176,15 @@ def _upgrade_genome_resources(galaxy_dir, base_url):
                 with open(local_file, "w") as out_handle:
                     out_handle.write(r.text)
 
-def _upgrade_snpeff_data(galaxy_dir, args):
+def _upgrade_snpeff_data(galaxy_dir, args, remotes):
     """Install or upgrade snpEff databases, localized to reference directory.
     """
     for dbkey, ref_file in genome.get_builds(galaxy_dir):
         resource_file = os.path.join(os.path.dirname(ref_file), "%s-resources.yaml" % dbkey)
         with open(resource_file) as in_handle:
             resources = yaml.load(in_handle)
-        snpeff_db, snpeff_base_dir = effects.get_db(ref_file, resources)
+        snpeff_db, snpeff_base_dir = effects.get_db({"resources": resources,
+                                                     "reference": {"fasta": {"base": ref_file}}})
         if snpeff_db:
             snpeff_db_dir = os.path.join(snpeff_base_dir, snpeff_db)
             if not os.path.exists(snpeff_db_dir):
@@ -157,8 +192,19 @@ def _upgrade_snpeff_data(galaxy_dir, args):
                 tooldir = args.tooldir or get_defaults()["tooldir"]
                 config = {"resources": {"snpeff": {"jvm_opts": ["-Xms500m", "-Xmx1g"],
                                                    "dir": os.path.join(tooldir, "share", "java", "snpeff")}}}
-                snpeff_cmd = effects.get_cmd("download", snpeff_base_dir, config)
-                subprocess.check_call("%s %s" % (snpeff_cmd, snpeff_db), shell=True)
+                raw_version = programs.java_versioner("snpeff", "snpEff",
+                                                      stdout_flag="snpEff version SnpEff")(config)
+                snpeff_version = "".join([x for x in raw_version
+                                          if x in set(string.digits + ".")]).replace(".", "_")
+                dl_url = remotes["snpeff_dl_url"].format(snpeff_ver=snpeff_version, genome=snpeff_db)
+                dl_file = os.path.basename(dl_url)
+                with utils.chdir(snpeff_base_dir):
+                    subprocess.check_call(["wget", "-c", "-O", dl_file, dl_url])
+                    subprocess.check_call(["unzip", dl_file])
+                    os.remove(dl_file)
+                dl_dir = os.path.join(snpeff_base_dir, "data", snpeff_db)
+                os.rename(dl_dir, snpeff_db_dir)
+                os.rmdir(os.path.join(snpeff_base_dir, "data"))
 
 def _get_biodata(base_file, args):
     with open(base_file) as in_handle:
@@ -170,6 +216,8 @@ def _get_biodata(base_file, args):
 
 def upgrade_thirdparty_tools(args, remotes):
     """Install and update third party tools used in the pipeline.
+
+    Creates a manifest directory with installed programs on the system.
     """
     s = {"fabricrc_overrides": {"system_install": args.tooldir,
                                 "local_install": os.path.join(args.tooldir, "local_install"),
@@ -186,6 +234,13 @@ def upgrade_thirdparty_tools(args, remotes):
     sys.path.insert(0, cbl["dir"])
     cbl_deploy = __import__("cloudbio.deploy", fromlist=["deploy"])
     cbl_deploy.deploy(s)
+    cbl_manifest = __import__("cloudbio.manifest", fromlist=["manifest"])
+    manifest_dir = os.path.join(_get_data_dir(), "manifest")
+    print("Creating manifest of installed packages in %s" % manifest_dir)
+    if os.path.exists(manifest_dir):
+        for fname in os.listdir(manifest_dir):
+            os.remove(os.path.join(manifest_dir, fname))
+    cbl_manifest.create(manifest_dir, args.tooldir)
 
 def _install_gemini(tooldir, datadir, args):
     """Install gemini layered on top of bcbio-nextgen, sharing anaconda framework.
@@ -206,7 +261,7 @@ def _install_gemini(tooldir, datadir, args):
         url = "https://raw.github.com/arq5x/gemini/master/gemini/scripts/gemini_install.py"
         script = os.path.basename(url)
         subprocess.check_call(["wget", "-O", script, url])
-        cmd = [sys.executable, script, tooldir, datadir, "--notools", "--nodata", "--sharedpy"]
+        cmd = [sys.executable, "-E", script, tooldir, datadir, "--notools", "--nodata", "--sharedpy"]
         if not args.sudo:
             cmd.append("--nosudo")
         subprocess.check_call(cmd)
@@ -259,10 +314,10 @@ def add_install_defaults(args):
     with open(install_config) as in_handle:
         default_args = yaml.load(in_handle)
     if default_args.get("tooldist") and args.tooldist == "minimal":
-        args.tooldist = default_args["tooldist"]
+        args.tooldist = str(default_args["tooldist"])
     if args.tools and args.tooldir is None:
         if "tooldir" in default_args:
-            args.tooldir = default_args["tooldir"]
+            args.tooldir = str(default_args["tooldir"])
         else:
             raise ValueError("Default tool directory not yet saved in config defaults. "
                              "Specify the '--tooldir=/path/to/tools' to upgrade tools. "
@@ -272,7 +327,7 @@ def add_install_defaults(args):
         for x in default_args.get(attr, []):
             new_val = getattr(args, attr)
             if x not in getattr(args, attr):
-                new_val.append(x)
+                new_val.append(str(x))
             setattr(args, attr, new_val)
     if "sudo" in default_args and not args.sudo is False:
         args.sudo = default_args["sudo"]
@@ -303,7 +358,7 @@ def add_subparser(subparsers):
                         action="append", default=["GRCh37"],
                         choices=["GRCh37", "hg19", "mm10", "mm9", "rn5", "canFam3"])
     parser.add_argument("--aligners", help="Aligner indexes to download",
-                        action="append", default=["bwa"],
+                        action="append", default=["bwa", "bowtie2"],
                         choices=["bowtie", "bowtie2", "bwa", "novoalign", "star", "ucsc"])
     parser.add_argument("--data", help="Upgrade data dependencies",
                         dest="install_data", action="store_true", default=False)
