@@ -42,29 +42,84 @@ def update_w_custom(config, lane_info):
 
 # ## Retrieval functions
 
-def load_system_config(config_file):
+def load_system_config(config_file, work_dir=None):
     """Load bcbio_system.yaml configuration file, handling standard defaults.
 
     Looks for configuration file in default location within
     final base directory from a standard installation. Handles both standard
     installs (galaxy/bcbio_system.yaml) and docker installs (config/bcbio_system.yaml).
     """
+    docker_config = _get_docker_config()
     if not os.path.exists(config_file):
-        base_dir = os.path.normpath(os.path.join(os.path.realpath(sys.executable), os.pardir, os.pardir, os.pardir))
-        docker_config = os.path.join(base_dir, "config", config_file)
+        base_dir = get_base_installdir()
         test_config = os.path.join(base_dir, "galaxy", config_file)
-        if os.path.exists(docker_config):
-            config_file = docker_config
-        elif os.path.exists(test_config):
+        if os.path.exists(test_config):
             config_file = test_config
         else:
             raise ValueError("Could not find input system configuration file %s, "
                              "including inside standard directory %s" %
                              (config_file, os.path.join(base_dir, "galaxy")))
     config = load_config(config_file)
+    if docker_config:
+        assert work_dir is not None, "Need working directory to merge docker config"
+        config_file = os.path.join(work_dir, "%s-merged%s" % os.path.splitext(os.path.basename(config_file)))
+        config = _merge_system_configs(config, docker_config, config_file)
+    if "algorithm" not in config:
+        config["algorithm"] = {}
     config["bcbio_system"] = config_file
     return config, config_file
 
+def get_base_installdir():
+    return os.path.normpath(os.path.join(os.path.realpath(sys.executable), os.pardir, os.pardir, os.pardir))
+
+def _merge_system_configs(host_config, container_config, out_file=None):
+    """Create a merged system configuration from external and internal specification.
+    """
+    out = copy.deepcopy(container_config)
+    for k, v in host_config.iteritems():
+        if k in set(["galaxy_config"]):
+            out[k] = v
+        elif k == "resources":
+            for pname, resources in v.iteritems():
+                if not isinstance(resources, dict) and pname not in out[k]:
+                    out[k][pname] = resources
+                else:
+                    for rname, rval in resources.iteritems():
+                        if rname in set(["cores", "jvm_opts", "memory"]):
+                            if pname not in out[k]:
+                                out[k][pname] = {}
+                            out[k][pname][rname] = rval
+    # Ensure final file is relocatable by mapping back to reference directory
+    if "bcbio_system" in out and ("galaxy_config" not in out or not os.path.isabs(out["galaxy_config"])):
+        out["galaxy_config"] = os.path.normpath(os.path.join(os.path.dirname(out["bcbio_system"]),
+                                                             os.pardir, "galaxy",
+                                                             "universe_wsgi.ini"))
+    if out_file:
+        with open(out_file, "w") as out_handle:
+            yaml.dump(out, out_handle, default_flow_style=False, allow_unicode=False)
+    return out
+
+def _get_docker_config():
+    base_dir = get_base_installdir()
+    docker_configfile = os.path.join(base_dir, "config", "bcbio_system.yaml")
+    if os.path.exists(docker_configfile):
+        return load_config(docker_configfile)
+
+def merge_resources(args):
+    """Merge docker local resources and global resource specification in a set of arguments.
+
+    Finds the `data` object within passed arguments and updates the resources
+    from a local docker configuration if present.
+    """
+    docker_config = _get_docker_config()
+    if not docker_config:
+        return args
+    else:
+        def _update_resources(config):
+            config["resources"] = _merge_system_configs(config, docker_config)["resources"]
+            return config
+        return _update_config(args, _update_resources)
+
 def load_config(config_file):
     """Load YAML config file, replacing environmental variables.
     """
@@ -115,7 +170,7 @@ def get_program(name, config, ptype="cmd", default=None):
     old_config = config.get("program", {}).get(name, None)
     if old_config:
         for key in ["dir", "cmd"]:
-            if not pconfig.has_key(key):
+            if not key in pconfig:
                 pconfig[key] = old_config
     if ptype == "cmd":
         return _get_program_cmd(name, pconfig, default)
@@ -146,7 +201,7 @@ def _get_program_cmd(name, config, default):
         return name
     elif isinstance(config, basestring):
         return config
-    elif config.has_key("cmd"):
+    elif "cmd" in config:
         return config["cmd"]
     elif default is not None:
         return default
@@ -160,7 +215,7 @@ def _get_program_dir(name, config):
         raise ValueError("Could not find directory in config for %s" % name)
     elif isinstance(config, basestring):
         return config
-    elif config.has_key("dir"):
+    elif "dir" in config:
         return expand_path(config["dir"])
     else:
         raise ValueError("Could not find directory in config for %s" % name)
@@ -169,6 +224,7 @@ def get_jar(base_name, dname):
     """Retrieve a jar in the provided directory
     """
     jars = glob.glob(os.path.join(expand_path(dname), "%s*.jar" % base_name))
+
     if len(jars) == 1:
         return jars[0]
     elif len(jars) > 1:
@@ -178,6 +234,66 @@ def get_jar(base_name, dname):
         raise ValueError("Could not find java jar %s in %s" %
                          (base_name, dname))
 
+# ## Retrieval and update to configuration from arguments
+
+def _dictdissoc(orig, k):
+    """Imitates immutability: create a new dictionary with the key dropped.
+    """
+    v = orig.pop(k, None)
+    new = copy.deepcopy(orig)
+    orig[k] = v
+    return new
+
+def is_std_config_arg(x):
+    return isinstance(x, dict) and "algorithm" in x and "resources" in x and not "files" in x
+
+def is_nested_config_arg(x):
+    return isinstance(x, dict) and "config" in x and is_std_config_arg(x["config"])
+
+def get_algorithm_config(xs):
+    """Flexibly extract algorithm configuration for a sample from any function arguments.
+    """
+    for x in xs:
+        if is_std_config_arg(x):
+            return x["algorithm"]
+        elif is_nested_config_arg(x):
+            return x["config"]["algorithm"]
+    raise ValueError("Did not find algorithm configuration in items: {0}"
+                     .format(xs))
+
+def add_cores_to_config(args, cores_per_job, parallel=None):
+    """Add information about available cores for a job to configuration.
+    Ugly hack to update core information in a configuration dictionary.
+    """
+    def _update_cores(config):
+        config["algorithm"]["num_cores"] = int(cores_per_job)
+        if parallel:
+            config["parallel"] = _dictdissoc(parallel, "view")
+        return config
+    return _update_config(args, _update_cores)
+
+def _update_config(args, update_fn):
+    """Update configuration, nested in argument list, with the provided update function.
+    """
+    new_i = None
+    for i, arg in enumerate(args):
+        if is_std_config_arg(arg) or is_nested_config_arg(arg):
+            new_i = i
+            break
+    if new_i is None:
+        raise ValueError("Could not find configuration in args: %s" % args)
+
+    new_arg = copy.deepcopy(args[new_i])
+    if is_nested_config_arg(new_arg):
+        new_arg["config"] = update_fn(new_arg["config"])
+    elif is_std_config_arg(new_arg):
+        new_arg = update_fn(new_arg)
+    else:
+        raise ValueError("Unexpected configuration dictionary: %s" % new_arg)
+    args = list(args)[:]
+    args[new_i] = new_arg
+    return args
+
 def adjust_memory(val, magnitude, direction="increase"):
     """Adjust memory based on number of cores utilized.
     """
@@ -234,7 +350,7 @@ def use_vqsr(algs):
         callers = alg.get("variantcaller", "gatk")
         if isinstance(callers, basestring):
             callers = [callers]
-        elif not callers: # no variant calling, no VQSR
+        elif not callers:  # no variant calling, no VQSR
             continue
         vqsr_supported_caller = False
         for c in callers:
@@ -242,7 +358,7 @@ def use_vqsr(algs):
                 vqsr_supported_caller = True
                 break
         if (cov.interval not in ["regional", "exome"] and cov.depth != "low"
-            and vqsr_supported_caller):
+              and vqsr_supported_caller):
             return True
     return False
 
