@@ -1,6 +1,7 @@
 """Read genome build configurations from Galaxy *.loc and bcbio-nextgen resource files.
 """
 import ConfigParser
+import glob
 import os
 from xml.etree import ElementTree
 
@@ -153,25 +154,21 @@ def _get_galaxy_data_table(name, dt_config_file):
 def get_refs(genome_build, aligner, galaxy_base):
     """Retrieve the reference genome file location from galaxy configuration.
     """
-    if not genome_build:
-        return (None, None)
-    galaxy_config = _get_galaxy_tool_info(galaxy_base)
-    out_info = []
-    for name in [aligner, "samtools"]:
-        if not name:
-            out_info.append(None)
-            continue
-        galaxy_dt = _get_galaxy_data_table(name, galaxy_config["tool_data_table_config_path"])
-        loc_file, need_remap = _get_galaxy_loc_file(name, galaxy_dt, galaxy_config["tool_data_path"],
-                                                    galaxy_base)
-        cur_ref = _get_ref_from_galaxy_loc(name, genome_build, loc_file, galaxy_dt, need_remap)
-        out_info.append(utils.add_full_path(cur_ref, galaxy_config["tool_data_path"]))
-
-    if len(out_info) != 2:
-        raise ValueError("Did not find genome reference for %s %s" %
-                         (genome_build, aligner))
-    else:
-        return tuple(out_info)
+    out = {}
+    name_remap = {"samtools": "fasta"}
+    if genome_build:
+        galaxy_config = _get_galaxy_tool_info(galaxy_base)
+        for name in [x for x in (aligner, "samtools") if x]:
+            galaxy_dt = _get_galaxy_data_table(name, galaxy_config["tool_data_table_config_path"])
+            loc_file, need_remap = _get_galaxy_loc_file(name, galaxy_dt, galaxy_config["tool_data_path"],
+                                                        galaxy_base)
+            cur_ref = _get_ref_from_galaxy_loc(name, genome_build, loc_file, galaxy_dt, need_remap)
+            base = os.path.normpath(utils.add_full_path(cur_ref, galaxy_config["tool_data_path"]))
+            indexes = glob.glob("%s*" % utils.splitext_plus(base)[0])
+            if base in indexes:
+                indexes.remove(base)
+            out[name_remap.get(name, name)] = {"base": base, "indexes": indexes}
+    return out
 
 def get_builds(galaxy_base):
     """Retrieve configured genome builds and reference files, using Galaxy configuration files.
@@ -183,4 +180,3 @@ def get_builds(galaxy_base):
                                                 galaxy_base)
     assert not need_remap, "Should not need to remap reference files"
     return _galaxy_loc_iter(loc_file, galaxy_dt)
-
