@@ -10,32 +10,14 @@ import pysam
 from bcbio import utils, broad
 from bcbio.log import logger
 from bcbio.bam import callable, ref
-from bcbio.bam.trim import brun_trim_fastq, trim_read_through
-from bcbio.pipeline.fastq import get_fastq_files, needs_fastq_conversion
+from bcbio.bam.trim import trim_read_through, trim_adapters
+from bcbio.pipeline.fastq import get_fastq_files
 from bcbio.pipeline.alignment import align_to_sort_bam
 from bcbio.pipeline import cleanbam
 from bcbio.variation import recalibrate
 from bcbio import bam
 from bcbio.bam import fastq
 
-
-def _item_needs_compute(lane_items):
-    """Determine if any item needs computing resources to spin up a cluster.
-    """
-    for item in lane_items:
-        if needs_fastq_conversion(item, item["config"]):
-            return True
-    return False
-
-def process_all_lanes(lanes, run_parallel):
-    """Process all input lanes, avoiding starting a cluster if not needed.
-    """
-    lanes = list(lanes)
-    if _item_needs_compute(lanes):
-        return run_parallel("process_lane", [[x] for x in lanes])
-    else:
-        return [process_lane(x)[0] for x in lanes]
-
 def process_lane(item):
     """Prepare lanes, potentially splitting based on barcodes and reducing the
     number of reads for a test run
@@ -46,18 +28,16 @@ def process_lane(item):
     if item.get("test_run", False):
         if bam.is_bam(file1):
             file1 = bam.downsample(file1, item, NUM_DOWNSAMPLE)
+            file2 = None
         else:
             file1, file2 = fastq.downsample(file1, file2, item,
                                             NUM_DOWNSAMPLE, quick=True)
-    item["files"] = (file1, file2)
-    return [item]
+    item["files"] = [file1, file2]
+    return [[item]]
 
 def trim_lane(item):
-    """
-    if trim_reads is set with no trimmer specified, default to B-run trimming
-    only. if trimmer is set to a supported type, perform that trimming
-    instead.
-
+    """Trim reads with the provided trimming method.
+    Support methods: read_through.
     """
     to_trim = [x for x in item["files"] if x is not None]
     dirs = item["dirs"]
@@ -68,22 +48,10 @@ def trim_lane(item):
         logger.info("Skipping trimming of %s." % (", ".join(to_trim)))
         return [[item]]
 
-    # swap the default to None if trim_reads gets deprecated
-
-    if trim_reads == "low_quality" or trim_reads == "true":
-        logger.info("Trimming low quality ends from %s."
-                    % (", ".join(to_trim)))
-        out_files = brun_trim_fastq(to_trim, dirs, config)
-
     if trim_reads == "read_through":
         logger.info("Trimming low quality ends and read through adapter "
                     "sequence from %s." % (", ".join(to_trim)))
-        out_files = trim_read_through(to_trim, dirs, config)
-
-    else:
-        logger.info("Trimming low quality ends from %s."
-                    % (", ".join(to_trim)))
-        out_files = brun_trim_fastq(to_trim, dirs, config)
+        out_files = trim_adapters(to_trim, dirs, config)
     item["files"] = out_files
     return [[item]]
 
@@ -93,15 +61,9 @@ def link_bam_file(orig_file, new_dir):
     """Provide symlinks of BAM file and existing indexes.
     """
     new_dir = utils.safe_makedir(new_dir)
-    update_files = []
-    for fname in (orig_file, "%s.bai" % orig_file,
-                  "%s.bai" % os.path.splitext(orig_file)[0]):
-        if utils.file_exists(fname):
-            sym_file = os.path.join(new_dir, os.path.basename(fname))
-            if not os.path.exists(sym_file):
-                os.symlink(fname, sym_file)
-            update_files.append(sym_file)
-    return update_files[0]
+    sym_file = os.path.join(new_dir, os.path.basename(orig_file))
+    utils.symlink_plus(orig_file, sym_file)
+    return sym_file
 
 def _check_prealigned_bam(in_bam, ref_file, config):
     """Ensure a pre-aligned BAM file matches the expected reference genome.
@@ -129,7 +91,9 @@ def _check_prealigned_bam(in_bam, ref_file, config):
 def process_alignment(data):
     """Do an alignment of fastq files, preparing a sorted BAM output file.
     """
-    if len(data["files"]) == 2:
+    if "files" not in data:
+        fastq1, fastq2 = None, None
+    elif len(data["files"]) == 2:
         fastq1, fastq2 = data["files"]
     else:
         assert len(data["files"]) == 1, data["files"]
@@ -186,4 +150,3 @@ def _recal_no_markduplicates(data):
     data = recalibrate.prep_recal(data)[0][0]
     data["config"] = orig_config
     return data
-
