@@ -4,6 +4,7 @@ Handles running the full pipeline based on instructions
 """
 import abc
 from collections import defaultdict
+import copy
 import os
 import sys
 import argparse
@@ -12,44 +13,36 @@ import tempfile
 
 from bcbio import install, log, structural, utils, upload
 from bcbio.bam import callable
-from bcbio.distributed.messaging import parallel_runner
-from bcbio.distributed.ipython import global_parallel
+from bcbio.distributed import clargs, prun, runfn
 from bcbio.log import logger
 from bcbio.ngsalign import alignprep
-from bcbio.pipeline import (disambiguate, lane, region, run_info, qcsummary, version,
-                            rnaseq)
+from bcbio.pipeline import (disambiguate, region, run_info, qcsummary,
+                            version, rnaseq)
 from bcbio.pipeline.config_utils import load_system_config
 from bcbio.provenance import programs, system, versioncheck
 from bcbio.server import main as server_main
 from bcbio.solexa.flowcell import get_fastq_dir
-from bcbio.variation.realign import parallel_realign_sample
-from bcbio.variation.genotype import parallel_variantcall, combine_multiple_callers
-from bcbio.variation import coverage, ensemble, population, recalibrate, validate
-from bcbio.rnaseq.count import combine_count_files
+from bcbio.variation.genotype import combine_multiple_callers
+from bcbio.variation import coverage, ensemble, population, validate
+from bcbio.rnaseq.count import (combine_count_files,
+                                annotate_combined_count_file)
 
-def run_main(work_dir, config_file=None, fc_dir=None, run_info_yaml=None,
-             numcores=None, paralleltype=None, queue=None, scheduler=None,
-             upgrade=None, profile=None, workflow=None, inputs=None,
-             resources="", timeout=15, retries=None):
+def run_main(workdir, config_file=None, fc_dir=None, run_info_yaml=None,
+             parallel=None, workflow=None):
     """Run variant analysis, handling command line options.
     """
-    config, config_file = load_system_config(config_file)
+    os.chdir(workdir)
+    config, config_file = load_system_config(config_file, workdir)
     if config.get("log_dir", None) is None:
-        config["log_dir"] = os.path.join(work_dir, "log")
-    paralleltype, numcores = _get_cores_and_type(numcores, paralleltype, scheduler)
-    parallel = {"type": paralleltype, "cores": numcores,
-                "scheduler": scheduler, "queue": queue,
-                "profile": profile, "module": "bcbio.distributed",
-                "resources": resources, "timeout": timeout,
-                "retries": retries}
+        config["log_dir"] = os.path.join(workdir, "log")
     if parallel["type"] in ["local"]:
         _setup_resources()
-        _run_toplevel(config, config_file, work_dir, parallel,
+        _run_toplevel(config, config_file, workdir, parallel,
                       fc_dir, run_info_yaml)
     elif parallel["type"] == "ipython":
         assert parallel["queue"] is not None, "IPython parallel requires a specified queue (-q)"
         assert parallel["scheduler"] is not None, "IPython parallel requires a specified scheduler (-s)"
-        _run_toplevel(config, config_file, work_dir, parallel,
+        _run_toplevel(config, config_file, workdir, parallel,
                       fc_dir, run_info_yaml)
     else:
         raise ValueError("Unexpected type of parallel run: %s" % parallel["type"])
@@ -68,17 +61,6 @@ def _setup_resources():
     target_hdls = min(max_hdls, target_procs)
     resource.setrlimit(resource.RLIMIT_NOFILE, (max(cur_hdls, target_hdls), max_hdls))
 
-def _get_cores_and_type(numcores, paralleltype, scheduler):
-    """Return core and parallelization approach from command line providing sane defaults.
-    """
-    if scheduler is not None:
-        paralleltype = "ipython"
-    if paralleltype is None:
-        paralleltype = "local"
-    if numcores is None:
-        numcores = 1
-    return paralleltype, int(numcores)
-
 def _run_toplevel(config, config_file, work_dir, parallel,
                   fc_dir=None, run_info_yaml=None):
     """
@@ -95,26 +77,22 @@ def _run_toplevel(config, config_file, work_dir, parallel,
     config_file = os.path.join(config_dir, os.path.basename(config_file))
     dirs = {"fastq": fastq_dir, "galaxy": galaxy_dir,
             "work": work_dir, "flowcell": fc_dir, "config": config_dir}
-    run_items = run_info.organize(dirs, config, run_info_yaml)
-    run_parallel = parallel_runner(parallel, dirs, config, config_file)
-
-    # process each flowcell lane
-    lane_items = lane.process_all_lanes(run_items, run_parallel)
-    pipelines = _pair_lanes_with_pipelines(lane_items)
+    samples = run_info.organize(dirs, config, run_info_yaml)
+    pipelines = _pair_lanes_with_pipelines(samples)
     final = []
     with utils.curdir_tmpdir() as tmpdir:
         tempfile.tempdir = tmpdir
         for pipeline, pipeline_items in pipelines.items():
-            pipeline_items = _add_provenance(pipeline_items, dirs, run_parallel, parallel, config)
+            pipeline_items = _add_provenance(pipeline_items, dirs, parallel, config)
             versioncheck.testall(pipeline_items)
-            for xs in pipeline.run(config, config_file, run_parallel, parallel, dirs, pipeline_items):
+            for xs in pipeline.run(config, config_file, parallel, dirs, pipeline_items):
                 if len(xs) == 1:
                     upload.from_sample(xs[0])
                     final.append(xs[0])
 
-def _add_provenance(items, dirs, run_parallel, parallel, config):
-    p = programs.write_versions(dirs, config)
-    system.write_info(dirs, run_parallel, parallel, config)
+def _add_provenance(items, dirs, parallel, config):
+    p = programs.write_versions(dirs, config, is_wrapper=parallel.get("wrapper") is not None)
+    system.write_info(dirs, parallel, config)
     out = []
     for item in items:
         if item.get("upload") and item["upload"].get("fc_name"):
@@ -147,13 +125,15 @@ def parse_cl_args(in_args):
     Returns the main config file and set of kwargs.
     """
     sub_cmds = {"upgrade": install.add_subparser,
-                "server": server_main.add_subparser}
+                "server": server_main.add_subparser,
+                "runfn": runfn.add_subparser,
+                "version": programs.add_subparser}
     parser = argparse.ArgumentParser(
         description="Best-practice pipelines for fully automated high throughput sequencing analysis.")
     sub_cmd = None
     if len(in_args) > 0 and in_args[0] in sub_cmds:
         subparsers = parser.add_subparsers(help="bcbio-nextgen supplemental commands")
-        sub_parser = sub_cmds[in_args[0]](subparsers)
+        sub_cmds[in_args[0]](subparsers)
         sub_cmd = in_args[0]
     else:
         parser.add_argument("global_config", help="Global YAML configuration file specifying details "
@@ -173,7 +153,7 @@ def parse_cl_args(in_args):
         parser.add_argument("-q", "--queue", help="Scheduler queue to run jobs on, for ipython parallel")
         parser.add_argument("-r", "--resources",
                             help=("Cluster specific resources specifications. Can be specified multiple times.\n"
-                                  "Supports SGE and SLURM parameters."),
+                                  "Supports SGE, Torque, LSF and SLURM parameters."),
                             default=[], action="append")
         parser.add_argument("--timeout", help="Number of minutes before cluster startup times out. Defaults to 15",
                             default=15, type=int)
@@ -181,9 +161,11 @@ def parse_cl_args(in_args):
                             help=("Number of retries of failed tasks during distributed processing. "
                                   "Default 0 (no retries)"),
                             default=0, type=int)
-        parser.add_argument("-p", "--profile", help="Profile name to use for ipython parallel",
-                            default="bcbio_nextgen")
+        parser.add_argument("-p", "--tag", help="Tag name to label jobs on the cluster",
+                            default="")
         parser.add_argument("-w", "--workflow", help="Run a workflow with the given commandline arguments")
+        parser.add_argument("--workdir", help="Directory to process in. Defaults to current working directory",
+                            default=os.getcwd())
         parser.add_argument("-v", "--version", help="Print current version",
                             action="store_true")
     args = parser.parse_args(in_args)
@@ -191,22 +173,12 @@ def parse_cl_args(in_args):
         error_msg = _sanity_check_args(args)
         if error_msg:
             parser.error(error_msg)
-        kwargs = {"numcores": args.numcores if args.numcores > 0 else None,
-                  "paralleltype": args.paralleltype,
-                  "scheduler": args.scheduler,
-                  "queue": args.queue,
-                  "timeout": args.timeout,
-                  "retries": args.retries,
-                  "resources": args.resources,
-                  "profile": args.profile,
-                  "workflow": args.workflow}
+        kwargs = {"parallel": clargs.to_parallel(args),
+                  "workflow": args.workflow,
+                  "workdir": args.workdir}
         kwargs = _add_inputs_to_kwargs(args, kwargs, parser)
     else:
         assert sub_cmd is not None
-        if sub_cmd == "upgrade":
-            # Nothing specified to do, print help
-            if not args.tools and not args.tooldir and not args.install_data and args.upgrade == "skip":
-                sub_parser.print_help()
         kwargs = {"args": args,
                   "config_file": None,
                   sub_cmd: True}
@@ -255,7 +227,8 @@ def _add_inputs_to_kwargs(args, kwargs, parser):
         fc_dir = os.path.abspath(fc_dir)
     if run_info_yaml:
         run_info_yaml = os.path.abspath(run_info_yaml)
-    kwargs["inputs"] = inputs
+    if kwargs.get("workflow"):
+        kwargs["inputs"] = inputs
     kwargs["config_file"] = global_config
     kwargs["fc_dir"] = fc_dir
     kwargs["run_info_yaml"] = run_info_yaml
@@ -273,6 +246,26 @@ def _get_full_paths(fastq_dir, config, config_file):
 
 # ## Generic pipeline framework
 
+def _wres(parallel, progs, fresources=None, ensure_mem=None):
+    """Add resource information to the parallel environment on required programs and files.
+
+    Enables spinning up required machines and operating in non-shared filesystem
+    environments.
+
+    progs -- Third party tools used in processing
+    fresources -- Required file-based resources needed. These will be transferred on non-shared
+                  filesystems.
+    ensure_mem -- Dictionary of required minimum memory for programs used. Ensures
+                  enough memory gets allocated on low-core machines.
+    """
+    parallel = copy.deepcopy(parallel)
+    parallel["progs"] = progs
+    if fresources:
+        parallel["fresources"] = fresources
+    if ensure_mem:
+        parallel["ensure_mem"] = ensure_mem
+    return parallel
+
 class AbstractPipeline:
     """
     Implement this class to participate in the Pipeline abstraction.
@@ -280,9 +273,7 @@ class AbstractPipeline:
         design:
             - analysis: name
     run: the steps run to perform the analyses
-
     """
-
     __metaclass__ = abc.ABCMeta
 
     @abc.abstractproperty
@@ -290,34 +281,9 @@ class AbstractPipeline:
         return
 
     @abc.abstractmethod
-    def run(self, config, config_file, run_parallel, parallel, dirs, lanes):
+    def run(self, config, config_file, parallel, dirs, lanes):
         return
 
-
-class VariantPipeline(AbstractPipeline):
-    name = "variant"
-
-    @classmethod
-    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
-        raise NotImplementedError("`variant` processing is deprecated: please use `variant2`"
-                                  "The next version will alias variant to the new variant2 pipeline")
-        lane_items = run_parallel("trim_lane", lane_items)
-        align_items = run_parallel("process_alignment", lane_items)
-        # process samples, potentially multiplexed across multiple lanes
-        samples = organize_samples(align_items, dirs, config_file)
-        samples = run_parallel("merge_sample", samples)
-        samples = run_parallel("prep_recal", samples)
-        samples = recalibrate.parallel_write_recal_bam(samples, run_parallel)
-        samples = parallel_realign_sample(samples, run_parallel)
-        samples = parallel_variantcall(samples, run_parallel)
-        samples = run_parallel("postprocess_variants", samples)
-        samples = combine_multiple_callers(samples)
-        samples = ensemble.combine_calls_parallel(samples, run_parallel)
-        samples = run_parallel("detect_sv", samples)
-        samples = qcsummary.generate_parallel(samples, run_parallel)
-        run_parallel("generate_bigwig", samples, {"programs": ["ucsc_bigwig"]})
-        return samples
-
 class Variant2Pipeline(AbstractPipeline):
     """Streamlined variant calling pipeline for large files.
     This is less generalized but faster in standard cases.
@@ -326,16 +292,18 @@ class Variant2Pipeline(AbstractPipeline):
     name = "variant2"
 
     @classmethod
-    def run(self, config, config_file, run_parallel, parallel, dirs, samples):
+    def run(self, config, config_file, parallel, dirs, samples):
         ## Alignment and preparation requiring the entire input file (multicore cluster)
-        with global_parallel(parallel, "multicore", ["process_alignment", "postprocess_alignment"],
-                             samples, dirs, config,
-                             multiplier=alignprep.parallel_multiplier(samples)) as parallel:
-            run_parallel = parallel_runner(parallel, dirs, config)
+        with prun.start(_wres(parallel, ["aligner", "gatk"],
+                              (["reference", "fasta"], ["reference", "aligner"], ["files"])),
+                        samples, config, dirs, "multicore",
+                        multiplier=alignprep.parallel_multiplier(samples)) as run_parallel:
             logger.info("Timing: alignment")
             samples = run_parallel("prep_align_inputs", samples)
+            samples = disambiguate.split(samples)
             samples = run_parallel("process_alignment", samples)
             samples = alignprep.merge_split_alignments(samples, run_parallel)
+            samples = disambiguate.resolve(samples, run_parallel)
             samples = run_parallel("postprocess_alignment", samples)
             regions = callable.combine_sample_regions(samples)
             samples = region.add_region_info(samples, regions)
@@ -344,33 +312,31 @@ class Variant2Pipeline(AbstractPipeline):
             samples = coverage.summarize_samples(samples, run_parallel)
 
         ## Variant calling on sub-regions of the input file (full cluster)
-        with global_parallel(parallel, "full", ["piped_bamprep", "variantcall_sample"],
-                             samples, dirs, config,
-                             multiplier=len(regions["analysis"]), max_multicore=1) as parallel:
-            run_parallel = parallel_runner(parallel, dirs, config)
+        with prun.start(_wres(parallel, ["gatk", "picard", "variantcaller"]),
+                        samples, config, dirs, "full",
+                        multiplier=len(regions["analysis"]), max_multicore=1) as run_parallel:
             logger.info("Timing: alignment post-processing")
             samples = region.parallel_prep_region(samples, regions, run_parallel)
             logger.info("Timing: variant calling")
             samples = region.parallel_variantcall_region(samples, run_parallel)
 
         ## Finalize variants (per-sample cluster)
-        with global_parallel(parallel, "persample", ["postprocess_variants"],
-                             samples, dirs, config) as parallel:
-            run_parallel = parallel_runner(parallel, dirs, config)
+        with prun.start(_wres(parallel, ["gatk", "gatk-vqsr", "snpeff", "bcbio_variation"]),
+                        samples, config, dirs, "persample") as run_parallel:
             logger.info("Timing: variant post-processing")
             samples = run_parallel("postprocess_variants", samples)
             logger.info("Timing: validation")
             samples = run_parallel("compare_to_rm", samples)
             samples = combine_multiple_callers(samples)
+        ## Finalizing BAMs and population databases, handle multicore computation
+        with prun.start(_wres(parallel, ["gemini", "samtools", "fastqc", "bamtools", "bcbio_variation",
+                                         "bcbio-variation-recall"]),
+                        samples, config, dirs, "multicore2") as run_parallel:
+            logger.info("Timing: prepped BAM merging")
+            samples = region.delayed_bamprep_merge(samples, run_parallel)
             logger.info("Timing: ensemble calling")
             samples = ensemble.combine_calls_parallel(samples, run_parallel)
             samples = validate.summarize_grading(samples)
-        ## Finalizing BAMs and population databases, handle multicore computation
-        with global_parallel(parallel, "multicore2", ["prep_gemini_db", "delayed_bam_merge"],
-                             samples, dirs, config) as parallel:
-            run_parallel = parallel_runner(parallel, dirs, config)
-            logger.info("Timing: prepped BAM merging")
-            samples = region.delayed_bamprep_merge(samples, run_parallel)
             logger.info("Timing: structural variation")
             samples = structural.run(samples, run_parallel)
             logger.info("Timing: population database")
@@ -380,27 +346,30 @@ class Variant2Pipeline(AbstractPipeline):
         logger.info("Timing: finished")
         return samples
 
-class SNPCallingPipeline(VariantPipeline):
+class SNPCallingPipeline(Variant2Pipeline):
     """Back compatible: old name for variant analysis.
     """
     name = "SNP calling"
 
+class VariantPipeline(Variant2Pipeline):
+    """Back compatibility; old name
+    """
+    name = "variant"
+
 class StandardPipeline(AbstractPipeline):
     """Minimal pipeline with alignment and QC.
     """
     name = "Standard"
     @classmethod
-    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
+    def run(self, config, config_file, parallel, dirs, lane_items):
         ## Alignment and preparation requiring the entire input file (multicore cluster)
-        with global_parallel(parallel, "multicore", ["process_alignment"],
-                             lane_items, dirs, config) as parallel:
-            run_parallel = parallel_runner(parallel, dirs, config)
+        with prun.start(_wres(parallel, ["aligner"]),
+                        lane_items, config, dirs, "multicore") as run_parallel:
             logger.info("Timing: alignment")
             samples = run_parallel("process_alignment", lane_items)
         ## Finalize (per-sample cluster)
-        with global_parallel(parallel, "persample", ["postprocess_variants"],
-                             samples, dirs, config) as parallel:
-            run_parallel = parallel_runner(parallel, dirs, config)
+        with prun.start(_wres(parallel, ["fastqc", "bamtools"]),
+                        samples, config, dirs, "persample") as run_parallel:
             logger.info("Timing: quality control")
             samples = qcsummary.generate_parallel(samples, run_parallel)
         logger.info("Timing: finished")
@@ -413,30 +382,54 @@ class RnaseqPipeline(AbstractPipeline):
     name = "RNA-seq"
 
     @classmethod
-    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
-        lane_items = run_parallel("trim_lane", lane_items)
-        samples = disambiguate.split(lane_items)
-        samples = run_parallel("process_alignment", samples)
-        samples = disambiguate.resolve(samples, run_parallel)
-        samples = rnaseq.estimate_expression(samples, run_parallel)
+    def run(self, config, config_file, parallel, dirs, samples):
+        with prun.start(_wres(parallel, ["picard"]),
+                        samples, config, dirs, "trimming") as run_parallel:
+            samples = run_parallel("process_lane", samples)
+            samples = run_parallel("trim_lane", samples)
+        with prun.start(_wres(parallel, ["aligner"],
+                              ensure_mem={"tophat": 8, "tophat2": 8, "star": 30}),
+                        samples, config, dirs, "multicore",
+                        multiplier=alignprep.parallel_multiplier(samples)) as run_parallel:
+            samples = disambiguate.split(samples)
+            samples = run_parallel("process_alignment", samples)
+            samples = disambiguate.resolve(samples, run_parallel)
+
+        with prun.start(_wres(parallel, ["samtools", "cufflinks"]),
+                        samples, config, dirs, "rnaseqcount") as run_parallel:
+            samples = rnaseq.estimate_expression(samples, run_parallel)
+            #samples = rnaseq.detect_fusion(samples, run_parallel)
+
         combined = combine_count_files([x[0].get("count_file") for x in samples])
+        gtf_file = utils.get_in(samples[0][0], ('genome_resources', 'rnaseq',
+                                                'transcripts'), None)
+        annotated = annotate_combined_count_file(combined, gtf_file)
         for x in samples:
             x[0]["combined_counts"] = combined
+            if annotated:
+                x[0]["annotated_combined_counts"] = annotated
 
-        samples = qcsummary.generate_parallel(samples, run_parallel)
-        #run_parallel("generate_bigwig", samples, {"programs": ["ucsc_bigwig"]})
+        with prun.start(_wres(parallel, ["picard", "fastqc", "rnaseqc"]),
+                        samples, config, dirs, "persample") as run_parallel:
+            samples = qcsummary.generate_parallel(samples, run_parallel)
         return samples
 
 class ChipseqPipeline(AbstractPipeline):
     name = "chip-seq"
 
     @classmethod
-    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
-        lane_items = run_parallel("trim_lane", lane_items)
-        samples = disambiguate.split(lane_items)
-        samples = run_parallel("process_alignment", samples)
-        samples = run_parallel("clean_chipseq_alignment", samples)
-        samples = qcsummary.generate_parallel(samples, run_parallel)
+    def run(self, config, config_file, parallel, dirs, samples):
+        with prun.start(_wres(parallel, ["aligner", "picard"]),
+                        samples, config, dirs, "multicore",
+                        multiplier=alignprep.parallel_multiplier(samples)) as run_parallel:
+            samples = run_parallel("process_lane", samples)
+            samples = run_parallel("trim_lane", samples)
+            samples = disambiguate.split(samples)
+            samples = run_parallel("process_alignment", samples)
+        with prun.start(_wres(parallel, ["picard", "fastqc"]),
+                        samples, config, dirs, "persample") as run_parallel:
+            samples = run_parallel("clean_chipseq_alignment", samples)
+            samples = qcsummary.generate_parallel(samples, run_parallel)
         return samples
 
 def _get_pipeline(item):
