@@ -5,7 +5,6 @@ import shutil
 import subprocess
 
 import lxml.html
-import pybedtools
 import yaml
 from datetime import datetime
 
@@ -88,6 +87,7 @@ def write_project_summary(samples):
                   if "dir" in samples[0][0]["upload"] else "")
     test_run = samples[0][0].get("test_run", False)
     date = str(datetime.now())
+    prev_samples = _other_pipeline_samples(out_file, samples)
     with open(out_file, "w") as out_handle:
         yaml.dump({"date": date}, out_handle,
                   default_flow_style=False, allow_unicode=False)
@@ -98,10 +98,22 @@ def write_project_summary(samples):
                   default_flow_style=False, allow_unicode=False)
         yaml.dump({"bcbio_system": samples[0][0]["config"].get("bcbio_system", "")}, out_handle,
                   default_flow_style=False, allow_unicode=False)
-        yaml.dump({"samples": [_save_fields(sample[0]) for sample in samples]}, out_handle,
+        yaml.dump({"samples": prev_samples + [_save_fields(sample[0]) for sample in samples]}, out_handle,
                   default_flow_style=False, allow_unicode=False)
     return out_file
 
+def _other_pipeline_samples(summary_file, cur_samples):
+    """Retrieve samples produced previously by another pipeline in the summary output.
+    """
+    cur_descriptions = set([s[0]["description"] for s in cur_samples])
+    out = []
+    if os.path.exists(summary_file):
+        with open(summary_file) as in_handle:
+            for s in yaml.load(in_handle).get("samples", []):
+                if s["description"] not in cur_descriptions:
+                    out.append(s)
+    return out
+
 def _save_fields(sample):
     to_save = ["dirs", "genome_resources", "genome_build", "sam_ref", "metadata",
                "description"]
@@ -113,21 +125,24 @@ def _save_fields(sample):
             if utils.file_exists(sample["disambiguate"]["summary"]):
                 disambigStats = _parse_disambiguate(sample["disambiguate"]["summary"])
                 saved["summary"]["metrics"]["Disambiguated %s reads" % str(sample["genome_build"])] = disambigStats[0]
-                saved["summary"]["metrics"]["Disambiguated %s reads" % str(sample["algorithm"]["disambiguate"][0])] = disambigStats[1]
+                disambigGenome = (sample["config"]["algorithm"]["disambiguate"][0]
+                                  if isinstance(sample["config"]["algorithm"]["disambiguate"], (list, tuple))
+                                  else sample["config"]["algorithm"]["disambiguate"])
+                saved["summary"]["metrics"]["Disambiguated %s reads" % disambigGenome] = disambigStats[1]
                 saved["summary"]["metrics"]["Disambiguated ambiguous reads"] = disambigStats[2]
     return saved
 
 def _parse_disambiguate(disambiguatestatsfilename):
     """Parse disambiguation stats from given file.
     """
-    disambigStats = [-1 -1 -1]
+    disambig_stats = [-1, -1, -1]
     with open(disambiguatestatsfilename, "r") as in_handle:
         header = in_handle.readline().strip().split("\t")
-        if header == ['sample','unique species A pairs','unique species B pairs','ambiguous pairs']:
-            disambigStatsTemp = in_handle.readline().strip().split("\t")[1:]
-            if len(disambigStatsTemp) == 3:
-                disambigStats = [int(x) for x in disambigStatsTemp]
-    return disambigStats
+        if header == ['sample', 'unique species A pairs', 'unique species B pairs', 'ambiguous pairs']:
+            disambig_stats_tmp = in_handle.readline().strip().split("\t")[1:]
+            if len(disambig_stats_tmp) == 3:
+                disambig_stats = [int(x) for x in disambig_stats_tmp]
+    return disambig_stats
 
 
 # ## Run and parse read information from FastQC
@@ -167,7 +182,8 @@ def _run_fastqc(bam_file, data, fastqc_out):
     Downsamples to 10 million reads to avoid excessive processing times with large
     files.
     """
-    if not os.path.exists(os.path.join(fastqc_out, "fastqc_report.html")):
+    sentry_file = os.path.join(fastqc_out, "fastqc_report.html")
+    if not os.path.exists(sentry_file):
         work_dir = os.path.dirname(fastqc_out)
         utils.safe_makedir(work_dir)
         ds_bam = bam.downsample(bam_file, data, 1e7)
@@ -182,7 +198,10 @@ def _run_fastqc(bam_file, data, fastqc_out):
                                              "%s_fastqc" % os.path.splitext(os.path.basename(bam_file))[0])
                 if os.path.exists("%s.zip" % fastqc_outdir):
                     os.remove("%s.zip" % fastqc_outdir)
-                shutil.move(fastqc_outdir, fastqc_out)
+                if not os.path.exists(sentry_file):
+                    if os.path.exists(fastqc_out):
+                        shutil.rmtree(fastqc_out)
+                    shutil.move(fastqc_outdir, fastqc_out)
         if ds_bam and os.path.exists(ds_bam):
             os.remove(ds_bam)
     parser = FastQCParser(fastqc_out)
@@ -190,10 +209,17 @@ def _run_fastqc(bam_file, data, fastqc_out):
     return stats
 
 def _run_complexity(bam_file, data, out_dir):
+    try:
+        import pandas as pd
+        import statsmodels.formula.api as sm
+    except ImportError:
+        return {"Unique Starts Per Read": "NA"}
+
+    SAMPLE_SIZE = 1000000
     base, _ = os.path.splitext(os.path.basename(bam_file))
     utils.safe_makedir(out_dir)
     out_file = os.path.join(out_dir, base + ".pdf")
-    df = bcbio.rnaseq.qc.starts_by_depth(bam_file, data["config"])
+    df = bcbio.rnaseq.qc.starts_by_depth(bam_file, data["config"], SAMPLE_SIZE)
     if not utils.file_exists(out_file):
         with file_transaction(out_file) as tmp_out_file:
             df.plot(x='reads', y='starts', title=bam_file + " complexity")
@@ -258,7 +284,7 @@ def _parse_qualimap_metrics(report_file):
     """Extract useful metrics from the qualimap HTML report file.
     """
     out = {}
-    parsers = {"Globals" : _parse_qualimap_globals,
+    parsers = {"Globals": _parse_qualimap_globals,
                "Globals (inside of regions)": _parse_qualimap_globals_inregion,
                "Coverage": _parse_qualimap_coverage,
                "Coverage (inside of regions)": _parse_qualimap_coverage,
@@ -274,12 +300,13 @@ def _parse_qualimap_metrics(report_file):
 def _bed_to_bed6(orig_file, out_dir):
     """Convert bed to required bed6 inputs.
     """
+    import pybedtools
     bed6_file = os.path.join(out_dir, "%s-bed6%s" % os.path.splitext(os.path.basename(orig_file)))
     if not utils.file_exists(bed6_file):
         with open(bed6_file, "w") as out_handle:
             for i, region in enumerate(list(x) for x in pybedtools.BedTool(orig_file)):
                 fillers = [str(i), "1.0", "+"]
-                full = region + fillers[:6-len(region)]
+                full = region + fillers[:6 - len(region)]
                 out_handle.write("\t".join(full) + "\n")
     return bed6_file
 
