@@ -1,4 +1,9 @@
-from bcbio.rnaseq import count, cufflinks
+from bcbio.rnaseq import featureCounts, cufflinks, oncofuse
+from bcbio.utils import get_in
+
+def detect_fusion(samples, run_parallel):
+    samples = run_parallel("run_oncofuse", samples)
+    return samples
 
 def estimate_expression(samples, run_parallel):
     samples = run_parallel("generate_transcript_counts", samples)
@@ -7,9 +12,12 @@ def estimate_expression(samples, run_parallel):
 
 def generate_transcript_counts(data):
     """Generate counts per transcript from an alignment"""
-    data["count_file"] = count.htseq_count(data)
+    data["count_file"] = featureCounts.count(data)
+    if get_in(data, ("config", "algorithm", "fusion_mode"), False):
+        data["oncofuse_file"] = oncofuse.run(data)
     return [[data]]
 
+
 def run_cufflinks(data):
     """Quantitate transcript expression with Cufflinks"""
     work_bam = data["work_bam"]
