@@ -3,7 +3,10 @@
 import os
 from contextlib import closing
 
-import pybedtools
+try:
+    import pybedtools
+except ImportError:
+    pybedtools = None
 import pysam
 
 from bcbio import bam, broad
@@ -118,10 +121,18 @@ def _rewrite_bed_with_chrom(in_file, out_file, chrom):
                 if line.startswith("%s\t" % chrom):
                     out_handle.write(line)
 
+
 def _subset_bed_by_region(in_file, out_file, region):
     orig_bed = pybedtools.BedTool(in_file)
     region_bed = pybedtools.BedTool("\t".join(str(x) for x in region) + "\n", from_string=True)
-    orig_bed.intersect(region_bed).saveas(out_file)
+    def _ensure_minsize(x):
+        while len(x) < 1:
+            if x.start > 0:
+                x.start -= 1
+            else:
+                x.end += 1
+        return x
+    orig_bed.intersect(region_bed).each(_ensure_minsize).merge().saveas(out_file)
 
 def subset_variant_regions(variant_regions, region, out_file):
     """Return BED file subset by a specified chromosome region.
