@@ -4,20 +4,25 @@ count number of reads mapping to features of transcripts
 """
 import os
 import sys
-import HTSeq
 import itertools
-import pandas as pd
 
-from bcbio.utils import (which, file_exists, get_in, safe_makedir)
+# soft imports
+try:
+    import HTSeq
+    import pandas as pd
+    import gffutils
+except ImportError:
+    HTSeq, pd, gffutils = None, None, None
+
+from bcbio.utils import (file_exists, get_in)
 from bcbio.distributed.transaction import file_transaction
-from bcbio.provenance import do
 from bcbio.log import logger
 from bcbio import bam
 
 
 def _get_files(data):
-#    in_file = _get_sam_file(data)
-    in_file = bam.sort(data["work_bam"], data["config"], order="queryname")
+    mapped = bam.mapped(data["work_bam"], data["config"])
+    in_file = bam.sort(mapped, data["config"], order="queryname")
     gtf_file = data["genome_resources"]["rnaseq"]["transcripts"]
     work_dir = data["dirs"].get("work", "work")
     out_dir = os.path.join(work_dir, "htseq-count")
@@ -38,14 +43,6 @@ def is_countfile(in_file):
     return True
 
 
-def _get_sam_file(data):
-    in_file = data["work_bam"]
-    config = data["config"]
-    sorted = bam.sort(in_file, config, "queryname")
-    sam = bam.bam_to_sam(sorted, config)
-    return sam
-
-
 def invert_strand(iv):
     iv2 = iv.copy()
     if iv2.strand == "+":
@@ -132,11 +129,8 @@ def htseq_count(data):
                          % feature_type)
 
     try:
-        if bam.is_sam(sam_filename):
-            read_seq = HTSeq.SAM_Reader(sam_filename)
-        elif bam.is_bam(sam_filename):
-            read_seq = HTSeq.BAM_Reader(sam_filename)
-        first_read = iter(read_seq).next()
+        align_reader = htseq_reader(sam_filename)
+        first_read = iter(align_reader).next()
         pe_mode = first_read.paired_end
     except:
         sys.stderr.write("Error occured when reading first line of sam "
@@ -145,8 +139,8 @@ def htseq_count(data):
 
     try:
         if pe_mode:
-            read_seq_pe_file = read_seq
-            read_seq = HTSeq.pair_SAM_alignments(read_seq)
+            read_seq_pe_file = align_reader
+            read_seq = HTSeq.pair_SAM_alignments(align_reader)
         empty = 0
         ambiguous = 0
         notaligned = 0
@@ -247,16 +241,20 @@ def htseq_count(data):
                 empty += 1
 
             if i % 100000 == 0:
-                sys.stderr.write("%d sam %s processed.\n" % ( i, "lines " if not pe_mode else "line pairs"))
+                sys.stderr.write("%d sam %s processed.\n" %
+                                 ( i, "lines " if not pe_mode else "line pairs"))
 
     except:
         if not pe_mode:
-            sys.stderr.write("Error occured in %s.\n" % read_seq.get_line_number_string())
+            sys.stderr.write("Error occured in %s.\n"
+                             % read_seq.get_line_number_string())
         else:
-            sys.stderr.write("Error occured in %s.\n" % read_seq_pe_file.get_line_number_string() )
+            sys.stderr.write("Error occured in %s.\n"
+                             % read_seq_pe_file.get_line_number_string() )
         raise
 
-    sys.stderr.write("%d sam %s processed.\n" % (i, "lines " if not pe_mode else "line pairs"))
+    sys.stderr.write("%d sam %s processed.\n" %
+                     (i, "lines " if not pe_mode else "line pairs"))
 
     with file_transaction(out_file) as tmp_out_file:
         with open(tmp_out_file, "w") as out_handle:
@@ -277,6 +275,9 @@ def htseq_count(data):
     return out_file
 
 def combine_count_files(files, out_file=None):
+    """
+    combine a set of count files into a single combined file
+    """
     for f in files:
         assert file_exists(f), "%s does not exist or is empty."
         assert is_countfile(f), "%s does not seem to be a count file."
@@ -295,8 +296,50 @@ def combine_count_files(files, out_file=None):
             df = pd.io.parsers.read_table(f, sep="\t", index_col=0, header=None,
                                           names=[col_names[0]])
         else:
-            df = df.join(pd.io.parsers.read_table(f, sep="\t", index_col=0, header=None,
+            df = df.join(pd.io.parsers.read_table(f, sep="\t", index_col=0,
+                                                  header=None,
                                                   names=[col_names[i]]))
 
     df.to_csv(out_file, sep="\t", index_label="id")
     return out_file
+
+def annotate_combined_count_file(count_file, gtf_file, out_file=None):
+    dbfn = gtf_file + ".db"
+    if not file_exists(dbfn):
+        return None
+
+    if not gffutils:
+        return None
+
+    db = gffutils.FeatureDB(dbfn, keep_order=True)
+
+    if not out_file:
+        out_dir = os.path.dirname(count_file)
+        out_file = os.path.join(out_dir, "annotated_combined.counts")
+
+    # if the genes don't have a gene_id or gene_name set, bail out
+    try:
+        symbol_lookup = {f['gene_id'][0]: f['gene_name'][0] for f in
+                         db.features_of_type('exon')}
+    except KeyError:
+        return None
+
+    df = pd.io.parsers.read_table(count_file, sep="\t", index_col=0, header=0)
+
+    df['symbol'] = df.apply(lambda x: symbol_lookup[x.name], axis=1)
+    df.to_csv(out_file, sep="\t", index_label="id")
+    return out_file
+
+
+def htseq_reader(align_file):
+    """
+    returns a read-by-read sequence reader for a BAM or SAM file
+    """
+    if bam.is_sam(align_file):
+        read_seq = HTSeq.SAM_Reader(align_file)
+    elif bam.is_bam(align_file):
+        read_seq = HTSeq.BAM_Reader(align_file)
+    else:
+        logger.error("%s is not a SAM or BAM file" % (align_file))
+        sys.exit(1)
+    return read_seq
