@@ -1,5 +1,6 @@
 """Provide ability to run bcbio-nextgen workflows.
 """
+import collections
 from functools import wraps
 import os
 import StringIO
@@ -11,7 +12,7 @@ import tornado.web
 import yaml
 
 from bcbio import utils
-from bcbio.pipeline import config_utils
+from bcbio.distributed import clargs
 
 def run_async(func):
     """Run function in an asychronous background thread.
@@ -44,29 +45,11 @@ def run_bcbio_nextgen(**kwargs):
     finally:
         print("Run ended: %s" % run_id)
 
-def _merge_system_configs(host_config, container_config, work_dir):
-    """Create a merged system configuration from external and internal specification.
-    """
-    out_file = os.path.join(work_dir, "web-bcbio_system.yaml")
-    out, _ = config_utils.load_system_config(container_config)
-    for k, v in host_config.iteritems():
-        if k in set(["galaxy_config"]):
-            out[k] = v
-        elif k == "resources":
-            for pname, resources in v.iteritems():
-                for rname, rval in resources.iteritems():
-                    if rname in set(["cores", "jvm_opts", "memory"]):
-                        if pname not in out[k]:
-                            out[k][pname] = {}
-                        out[k][pname][rname] = rval
-    # Ensure final file is relocatable by mapping back to reference directory
-    if "bcbio_system" in out and ("galaxy_config" not in out or not os.path.isabs(out["galaxy_config"])):
-        out["galaxy_config"] = os.path.normpath(os.path.join(os.path.dirname(out["bcbio_system"]),
-                                                             os.pardir, "galaxy",
-                                                             "universe_wsgi.ini"))
-    with open(out_file, "w") as out_handle:
-        yaml.dump(out, out_handle, default_flow_style=False, allow_unicode=False)
-    return out_file
+def _rargs_to_parallel_args(rargs):
+    Args = collections.namedtuple("Args", "numcores scheduler queue resources timeout retries")
+    return Args(int(rargs.get("numcores", 1)), rargs.get("scheduler"),
+                rargs.get("queue"), rargs.get("resources", ""),
+                int(rargs.get("timeout", 15)), rargs.get("retries"))
 
 def get_handler(args):
     class RunHandler(tornado.web.RequestHandler):
@@ -76,8 +59,9 @@ def get_handler(args):
             rargs = yaml.safe_load(StringIO.StringIO(str(self.get_argument("args", "{}"))))
             system_config = args.config or "bcbio_system.yaml"
             if "system_config" in rargs:
-                system_config = _merge_system_configs(rargs["system_config"],
-                                                      system_config, rargs["work_dir"])
+                system_config = os.path.join(rargs["work_dir"], "web-system_config.yaml")
+                with open(system_config, "w") as out_handle:
+                    yaml.dump(rargs["system_config"], out_handle, default_flow_style=False, allow_unicode=False)
             if "sample_config" in rargs:
                 sample_config = os.path.join(rargs["work_dir"], "web-sample_config.yaml")
                 with open(sample_config, "w") as out_handle:
@@ -88,12 +72,7 @@ def get_handler(args):
                       "config_file": system_config,
                       "run_info_yaml": sample_config,
                       "fc_dir": rargs.get("fc_dir"),
-                      "numcores": int(rargs.get("numcores", 1)),
-                      "scheduler": rargs.get("scheduler"),
-                      "queue": rargs.get("queue"),
-                      "resources": rargs.get("resources", ""),
-                      "timeout": int(rargs.get("timeout", 15)),
-                      "retries": rargs.get("retries"),
+                      "parallel": clargs.to_parallel(_rargs_to_parallel_args(rargs)),
                       "app": self.application}
             run_id = yield tornado.gen.Task(run_bcbio_nextgen, **kwargs)
             self.write(run_id)
