@@ -114,7 +114,7 @@ def _maybe_add_alignment(algorithm, sample, out):
                     "ext": "ready"})
         if file_exists(sample["work_bam"] + ".bai"):
             out.append({"path": sample["work_bam"] + ".bai",
-                        "type": "bai",
+                        "type": "bam.bai",
                         "ext": "ready"})
     return out
 
@@ -123,9 +123,10 @@ def _maybe_add_counts(algorithm, sample, out):
                 "type": "counts",
                 "ext": "ready"})
     stats_file = os.path.splitext(sample["count_file"])[0] + ".stats"
-    out.append({"path": stats_file,
-                "type": "count_stats",
-                "ext": "ready"})
+    if file_exists(stats_file):
+        out.append({"path": stats_file,
+                    "type": "count_stats",
+                    "ext": "ready"})
     return out
 
 def _maybe_add_cufflinks(algorithm, sample, out):
@@ -175,5 +176,7 @@ def _get_files_project(sample, upload_config):
 
     if "combined_counts" in sample:
         out.append({"path": sample["combined_counts"]})
+    if "annotated_combined_counts" in sample:
+        out.append({"path": sample["annotated_combined_counts"]})
 
     return _add_meta(out, config=upload_config)
