@@ -3,6 +3,7 @@
 import os
 import unittest
 
+from nose.plugins.attrib import attr
 import yaml
 
 from bcbio.solexa import samplesheet
@@ -14,6 +15,7 @@ class SampleSheetTest(unittest.TestCase):
         self.ss_file = os.path.join(os.path.dirname(__file__),
                                     "data", "illumina_samplesheet.csv")
 
+    @attr(speed=1)
     def test_toyaml(self):
         """Convert CSV Illumina SampleSheet to YAML.
         """
@@ -25,6 +27,7 @@ class SampleSheetTest(unittest.TestCase):
         assert info[0]['multiplex'][0]['barcode_id'] == 5
         os.remove(out_file)
 
+    @attr(speed=1)
     def test_checkforrun(self):
         """Check for the presence of runs in an Illumina SampleSheet.
         """
