@@ -46,6 +46,34 @@ def expected_failure(test):
             raise AssertionError('Failure expected')
     return inner
 
+def get_post_process_yaml(data_dir, workdir):
+    try:
+        from bcbiovm.docker.defaults import get_datadir
+        datadir = get_datadir()
+        system = os.path.join(datadir, "galaxy", "bcbio_system.yaml") if datadir else None
+    except ImportError:
+        system = None
+    if system is None or not os.path.exists(system):
+        try:
+            _, system = load_system_config("bcbio_system.yaml")
+        except ValueError:
+            system = None
+    sample = os.path.join(data_dir, "post_process-sample.yaml")
+    std = os.path.join(data_dir, "post_process.yaml")
+    if os.path.exists(std):
+        return std
+    elif system and os.path.exists(system):
+        # create local config pointing to reduced genomes
+        test_system = os.path.join(workdir, os.path.basename(system))
+        with open(system) as in_handle:
+            config = yaml.load(in_handle)
+            config["galaxy_config"] = os.path.join(data_dir, "universe_wsgi.ini")
+            with open(test_system, "w") as out_handle:
+                yaml.dump(config, out_handle)
+        return test_system
+    else:
+        return sample
+
 class AutomatedAnalysisTest(unittest.TestCase):
     """Setup a full automated analysis and run the pipeline.
     """
@@ -87,27 +115,6 @@ class AutomatedAnalysisTest(unittest.TestCase):
         os.rename(os.path.basename(dirname), dirname)
         os.remove(os.path.basename(url))
 
-    def _get_post_process_yaml(self, workdir):
-        std = os.path.join(self.data_dir, "post_process.yaml")
-        try:
-            _, system = load_system_config("bcbio_system.yaml")
-        except ValueError:
-            system = None
-        sample = os.path.join(self.data_dir, "post_process-sample.yaml")
-        if os.path.exists(std):
-            return std
-        elif system:
-            # create local config pointing to reduced genomes
-            test_system = os.path.join(workdir, os.path.basename(system))
-            with open(system) as in_handle:
-                config = yaml.load(in_handle)
-                config["galaxy_config"] = os.path.join(self.data_dir, "universe_wsgi.ini")
-                with open(test_system, "w") as out_handle:
-                    yaml.dump(config, out_handle)
-            return test_system
-        else:
-            return sample
-
     @attr(speed=3)
     def IGNOREtest_3_full_pipeline(self):
         """Run full automated analysis pipeline with multiplexing.
@@ -117,7 +124,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "110106_FC70BUKAAXX"),
                   os.path.join(self.data_dir, "run_info.yaml")]
             subprocess.check_call(cl)
@@ -130,7 +137,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         """
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "110221_empty_FC12345AAXX"),
                   os.path.join(self.data_dir, "run_info-empty.yaml")]
             subprocess.check_call(cl)
@@ -142,36 +149,51 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "test_stranded"),
                   os.path.join(self.data_dir, "run_info-stranded.yaml")]
             subprocess.check_call(cl)
 
     @attr(rnaseq=True)
+    @attr(tophat=True)
     def test_2_rnaseq(self):
         """Run an RNA-seq analysis with TopHat and generate gene-level counts.
         """
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "110907_ERP000591"),
                   os.path.join(self.data_dir, "run_info-rnaseq.yaml")]
             subprocess.check_call(cl)
 
+    @expected_failure
+    @attr(fusion=True)
+    def test_2_fusion(self):
+        """Run an RNA-seq analysis and test fusion genes
+        """
+        self._install_test_files(self.data_dir)
+        with make_workdir() as workdir:
+            cl = ["bcbio_nextgen.py",
+                  "/usr/local/share/bcbio-nextgen/galaxy/bcbio_system.yaml",
+                  # get_post_process_yaml(self.data_dir, workdir),
+                  os.path.join(self.data_dir, os.pardir, "test_fusion"),
+                  os.path.join(self.data_dir, "run_info-fusion.yaml")]
+            subprocess.check_call(cl)
+
     @attr(rnaseq=True)
+    @attr(star=True)
     def test_2_star(self):
         """Run an RNA-seq analysis with STAR and generate gene-level counts.
         """
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "110907_ERP000591"),
                   os.path.join(self.data_dir, "run_info-star.yaml")]
             subprocess.check_call(cl)
 
-
     @attr(explant=True)
     def test_explant(self):
         """
@@ -180,7 +202,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "1_explant"),
                   os.path.join(self.data_dir, "run_info-explant.yaml")]
             subprocess.check_call(cl)
@@ -193,25 +215,25 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "test_chipseq"),
                   os.path.join(self.data_dir, "run_info-chipseq.yaml")]
             subprocess.check_call(cl)
 
-
     @attr(speed=1)
+    @attr(ensemble=True)
     def test_1_variantcall(self):
         """Test variant calling with GATK pipeline.
         """
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "100326_FC6107FAAXX"),
                   os.path.join(self.data_dir, "run_info-variantcall.yaml")]
             subprocess.check_call(cl)
 
-    @attr(speed=2)
+    @attr(speed=1)
     @attr(devel=True)
     def test_5_bam(self):
         """Allow BAM files as input to pipeline.
@@ -219,7 +241,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "100326_FC6107FAAXX"),
                   os.path.join(self.data_dir, "run_info-bam.yaml")]
             subprocess.check_call(cl)
@@ -231,7 +253,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "100326_FC6107FAAXX"),
                   os.path.join(self.data_dir, "run_info-bamclean.yaml")]
             subprocess.check_call(cl)
@@ -244,11 +266,23 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  self._get_post_process_yaml(workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "tcga_benchmark"),
                   os.path.join(self.data_dir, "run_info-cancer.yaml")]
             subprocess.check_call(cl)
 
+    @attr(cancer=True)
+    def test_7_cancer_nonormal(self):
+        """Test cancer calling without normal samples or with normal VCF panels.
+        XXX Not yet working
+        """
+        self._install_test_files(self.data_dir)
+        with make_workdir() as workdir:
+            cl = ["bcbio_nextgen.py",
+                  get_post_process_yaml(self.data_dir, workdir),
+                  os.path.join(self.data_dir, "run_info-cancer2.yaml")]
+            subprocess.check_call(cl)
+
     @attr(speed=1)
     @attr(template=True)
     def test_8_template(self):
@@ -272,8 +306,23 @@ class AutomatedAnalysisTest(unittest.TestCase):
         """
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
-            cl = ["bcbio_nextgen_docker.py", "run",
-                  "--systemconfig=%s" % self._get_post_process_yaml(workdir),
+            cl = ["bcbio_vm.py", "run",
+                  "--systemconfig=%s" % get_post_process_yaml(self.data_dir, workdir),
                   "--fcdir=%s" % os.path.join(self.data_dir, os.pardir, "100326_FC6107FAAXX"),
                   os.path.join(self.data_dir, "run_info-bam.yaml")]
             subprocess.check_call(cl)
+
+    @attr(docker_ipython=True)
+    def test_docker_ipython(self):
+        """Run an analysis with code and tools inside a docker container, driven via IPython.
+
+        Requires https://github.com/chapmanb/bcbio-nextgen-vm
+        """
+        self._install_test_files(self.data_dir)
+        with make_workdir() as workdir:
+            cl = ["bcbio_vm.py", "ipython",
+                  "--systemconfig=%s" % get_post_process_yaml(self.data_dir, workdir),
+                  "--fcdir=%s" % os.path.join(self.data_dir, os.pardir, "100326_FC6107FAAXX"),
+                  os.path.join(self.data_dir, "run_info-bam.yaml"),
+                  "lsf", "localrun"]
+            subprocess.check_call(cl)
